import tkinter as tk
from tkinter import ttk
from tkinter import scrolledtext
from tkinter import Menu
from tkinter import messagebox as msg
from time import sleep

# 툴팁 생성 클래스
class ToolTip(object):
    def __init__(self,widget):
         self.widget=widget
         self.tip_window=None
     # 툴팁이 보이게 하는 메소드   
    def show_tip(self,tip_text):
        if self.tip_window or not tip_text:
            return
        x,y,_cx,cy=self.widget.bbox("insert") # 바운딩 박스,cx,cy는 폭과 높이, 문자를 표시하지 않으면 none을 반환
        x=x+self.widget.winfo_rootx()+25 # 부모 윈도우를 기준으로 해서 왼쪽의 x좌표와 윗면의  y좌표를 반환한다. 
        y=y+cy+self.widget.winfo_rooty()+25
        self.tip_window=tw=tk.Toplevel(self.widget)
        tw.wm_overrideredirect(True) #데코레이션을 해주냐, 안해주냐  예를 들어 확채,축소,x그런게 다 데코레이션이다.
        tw.wm_geometry("+%d+%d"%(x,y)) #윈도우가 놓일 위치를 제어한다. 
        label=tk.Label(tw,text=tip_text,justify=tk.LEFT,background="#ffffe0",relief=tk.SOLID,borderwidth=1,font=("tahomg","8","normal"))
        label.pack(ipadx=1)
    # 툴팁이 안보이게 하는 메소드    
    def hide_tip(self):
        tw=self.tip_window
        self.tip_window=None
        if tw:
            tw.destroy()
            
 # 툴팁 생성 클래스를 호출하는 함수 (콜백함수)               
def create_ToolTip(widget, text):
    toolTip=ToolTip(widget)
    def enter(event):
        toolTip.show_tip(text)
    def leave(event):
        toolTip.hide_tip()
    widget.bind("<Enter>",enter)
    widget.bind("<Leave>",leave)

def run_progressbar():
    progress_bar['maximum']=100
    for i in range(101):
        sleep(0.05)
        progress_bar['value']=i
        progress_bar.update()
    progress_bar['value']=0
    
def start_progressbar():
    progress_bar.start()
    
def stop_progressbar():
    progress_bar.stop()
    
def progressbar_stop_after(wait_ms=1000):
    win.after(wait_ms,progress_bar.stop) 
        
win=tk.Tk()
win.title("Python GUI")

# (help메뉴 콜백) 메시지박스 함수
def _msgBox():
    ret =msg.askyesno("Askyesno Messagr Box", "Is it correct?")
    if ret >0: # 왼쪽 버튼 클릭시 리턴값이 0초과인 듯 함
        print("RETRY")
    else:
        print("CANCEL")
        
# (버튼 콜백) 버튼 설정 변경 함수
def click_me():
    action.configure(text="Hello"+name.get()+''+number.get())
    
# (콤보박스 콜백) 스크롤드 텍스트 출력 함수
def print_Combobox(event):
        scr.insert(tk.INSERT, "The selected number of Combobox is "+number.get()+'.\n')
    
# (스핀 콜백) 스크롤드텍스트 출력 함수
def _spin():
    scr.insert(tk.INSERT, spin.get()+'\n')
    
# 메뉴 생성
menu_bar = Menu(win)
win.config(menu=menu_bar) 

# 메뉴 아이템 생성
file_menu = Menu(menu_bar,tearoff=0) # 메뉴안에 메뉴를 생성=>(메뉴 안) 아이템칸 생성
file_menu.add_command(label="New") # 아이템 추가
file_menu.add_command(label="Exit",command=win.quit) # 아이템 추가 및 입력 받을 시 콜백함수 호출
file_menu.add_separator()
menu_bar.add_cascade(label="File",menu=file_menu)

help_menu=Menu(menu_bar, tearoff=0)
menu_bar.add_cascade(label="Help",menu=help_menu)
help_menu.add_command(label="About",command=_msgBox)    


# 탭 생성에 필요한 탭 컨트롤 생성
tabControl=ttk.Notebook(win)

# 새로운 탭(탭1) 추가
tab1=ttk.Frame(tabControl)
tabControl.add(tab1, text="Tab1")

# 새로운 탭(탭2) 추가
tab2=ttk.Frame(tabControl)
tabControl.add(tab2, text="Tab2")

# grid메소드과  비슷, fill은 제공된 공간을 점유하도록 지정, expand는 여분의 공간을 채우기 위해 확장할건지
tabControl.pack(expand=1,fill="both") 

# tab1에 라벨프레임 생성
mighty= ttk.LabelFrame(tab1,text="Mighty python")
mighty.grid(column=0, row=0,padx=8,pady=4)


# (tab1-라벨프레임 내) 라벨 생성
a_label=ttk.Label(mighty,text="Enter a name:")
a_label.grid(column=0, row=0,sticky='W')

# (tab1-라벨프레임 내) 엔트리 위젯
name= tk.StringVar()
name_entered = ttk.Entry(mighty, width=12, textvariable=name)
name_entered.grid(column=0, row=1)
name_entered.focus()


# (tab1-라벨프레임 내) 버튼 위젯
action=ttk.Button(mighty, text="Click Me!", command=click_me)
action.grid(column=2, row=1)

ttk.Label(mighty,text="Choose a number").grid(column=1,row=0)

# (tab1-라벨프레임 내) 콤보박스 위젯
number=tk.StringVar()
number_chosen=ttk.Combobox(mighty,width=12,textvariable=number, state= 'readonly')
number_chosen['values']=(1,2,4,42,100)
number_chosen.grid(column=1,row=1)
number_chosen.current(0)
number_chosen.bind("<<ComboboxSelected>>",print_Combobox)

# (tab1-라벨프레임 내) 스핀박스
spin=tk.Spinbox(mighty,values=(1,2,4,42,100),width=5,bd=8,command=_spin)
spin.grid(column=0,row=2)
create_ToolTip(spin,'This is a spincontrol')

spin2=tk.Spinbox(mighty,values=(0,50,100),width=5,bd=9,relief=tk.FLAT) # relief는 스핀박스 테두리를 디자인하는 메소드
spin2.grid(column=1,row=2)

# (tab1-라벨프레임 내) 스크롤드 테스트  위젯
scrol_w = 30
scrol_h =3
scr = scrolledtext.ScrolledText(mighty, width=scrol_w, height=scrol_h, wrap=tk.WORD)
scr.grid(column=0, columnspan=3, row=3, sticky='WE')
create_ToolTip(scr,'This is a ScrolledText widget')


# (tab1-라벨프레임 내) 그리드 설정 변경
for child in mighty.winfo_children():
    child.grid_configure(sticky='W')
    
# tab2에 라벨프레임 생성
mighty2= ttk.LabelFrame(tab2,text="The Snake")
mighty2.grid(column=0, row=0,padx=8,pady=4)
    
def hi():
    print(chVarUn.get())
    
# (tab2-라벨프레임 내) 체크버튼 위젯
chVarDis=tk.IntVar()
check1 = tk.Checkbutton(mighty2, text="Disabled",variable=chVarDis,state="disabled")
check1.select()
check1.grid(column=0, row=0, sticky=tk.W)

chVarUn=tk.IntVar()
check2 = tk.Checkbutton(mighty2, text="UnChecked",variable=chVarUn)
check2.deselect()
check2.grid(column=1, row=0, sticky=tk.W)

chVarEn=tk.IntVar()
check3 = tk.Checkbutton(mighty2, text="Enabled",variable=chVarEn)
check3.select()
check3.grid(column=2, row=0, sticky=tk.W)

# (tab2-라벨프레임 내) 라디오버튼 위젯
colors=["Blue","Gold","Red"]

def radCall():
    redsel=radVar.get()
    mighty2.configure(text=colors[redsel]) # 라디오버튼 클릭시 라벨프레임 텍스트 변경


radVar = tk.IntVar()

radVar.set(99)

for col in range(3):
    curRad=tk.Radiobutton(mighty2,text=colors[col], variable =radVar,value=col, command=radCall)
    curRad.grid(column=col, row=1, sticky=tk.W)

# (tab2-라벨프레임 내) 라벨 프레임 위젯
buttons_frame = ttk.Labelframe(mighty2, text='ProgressBar')
buttons_frame.grid(column=0, row=2)

ttk.Button(buttons_frame,text="Run Progressbar",command=run_progressbar).grid(column=0,row=0,sticky='W')
ttk.Button(buttons_frame,text="Start Progressbar",command=start_progressbar).grid(column=0,row=1,sticky='W')
ttk.Button(buttons_frame,text="Stop Progressbar",command=stop_progressbar).grid(column=0,row=2,sticky='W')
ttk.Button(buttons_frame,text="Stop after second",command=lambda:progressbar_stop_after(2000)).grid(column=0,row=3,sticky='W')


progress_bar=ttk.Progressbar(tab2,orient='horizontal',length=286,mode='indeterminate')
progress_bar.grid(column=0,row=3,pady=2)


# 이벤트순환문 실행
win.mainloop()
